const express = require('express')
const http = require('http')
const path = require('path')
const fs = require('fs');
const dgram = require('dgram');
const dotenv  = require('dotenv');
const app = express()
const bodyParser = require('body-parser'); //

app.use(express.json({ limit : "50mb" })); 
app.use(express.urlencoded({ limit:"50mb", extended: false }));

dotenv.config({
    path: path.resolve(
      process.cwd(),
      process.env.NODE_ENV == "production" ? ".env" : ".env.dev"
    )
});

const server = dgram.createSocket('udp4');
// socket 실행
server.bind(9400);

server.on('listening', function() {
    console.log('Socket Port : 9400');
});


server.on('message', function(msg ) {

    udpResultMsg =  msg.toString();
    console.log("케켘", udpResultMsg)

});

server.on('close', function() {
    console.log('Server UDP Socket close');
});

const client = dgram.createSocket('udp4');
const clientPort= 9300;
//  const clientPort= 4403;

// const clientHost = '192.168.52.122'; //inet
const clientHost ='192.168.52.38';

// Error [ERR_SOCKET_DGRAM_NOT_RUNNING]: Not running 
client.on('close', function() {
    console.log('Client UDP socket close')
});

// const jsonFile = fs.readFileSync('./json/projects.json', 'utf8');
// console.log(jsonFile);

// const jsonData = JSON.parse(jsonFile);

// console.table(jsonData);

//Post 방식은 Get 과 다르기 때문에 body-parser 를 설치해서 사용해야한다.
app.use(bodyParser.json());
app.use(bodyParser.urlencoded({extended: false}))

//vue router 연동하기 위한 설정
app.use(require('connect-history-api-fallback')());

app.use(express.static(path.join(__dirname, 'public')));

app.set('port',process.env.PORT || 3001);

/* udp 통신으로 매니저에게 받는 결과값*/
let udpResultMsg = ''
/* udp 통신 기다리는 시간*/
let udpAwaitTime = 0

let newProjectNum = 0

app.get('/api/getWpoId',(req,res) => {
    let id = process.env.WPO
    res.send(id)
})

app.post('/api/projectInfo',async (req,res) => {
    const dataBuffer = fs.readFileSync('./json/project.json')
    const project = JSON.parse(dataBuffer.toString())
    console.log('로컬 프로젝트 :' , project)
    
    // 받아온 값이 있을경우
    if(Object.keys(req.body).length !== 0 ){
        project.admin = req.body.admin
        project.aoiUid = req.body.aoiUid
        project.counter = req.body.counter
        project.createDate = req.body.createDate
        project.name = req.body.name
        project.state = req.body.state
        project.successDate = req.body.successDate
        project.uid = req.body.uid
        project.user = req.body.user
        project.infos = req.body.infos
        project.updateDate = new Date()
        fs.writeFileSync('./json/project.json',JSON.stringify(project) , (err) =>{
            if ( err ) return err; 
        })
    }

    res.status(200).json(project)
    
})

const wrapper = asyncFn => { return (async (req, res, next) => { try { return await asyncFn(req, res, next); } catch (error) { return next(error); } }); };

//https://kjwsx23.tistory.com/199 express 와 async ,await 이슈
//express async await wrapper 함수를 써서 감싸거나 라이브러리를 써야 한다. 
app.get('/api/requestManager',wrapper(async (req,res)=>{
    
    req.query.image_path = process.env.TEACHING_IMAGE_PATH

    if(req.query.cmd === "neuro_check"){
        req.query.teaching = `${process.env.TEACHING_IMAGE_PATH}-${req.query.project_num}-2-${process.env.TEACHING_RESOLUTION}${req.query.teaching_info}` 
    } 

    console.log("param :",JSON.stringify(req.query) )

    sendToManager (JSON.stringify(req.query))

    let requestMsg = ''

    switch(req.query.cmd){
        case "neuro_start":
            requestMsg = 'neuro_start_done'
            break
        case "neuro_ready": 
            requestMsg = 'neuro_ready_done'
            break 
        case "neuro_capture": 
            requestMsg = 'capture_done'
            break 
        case "neuro_check" :
            requestMsg = 'neuro_result'
            break           
        // case "neuro_replay" :
        //     requestMsg = 'neuro_result'
        //     break           

        default:
            res.status(404)
    }

    await responseManager(requestMsg)

        .then(data => {
            res.status(200).json(data)
        })
        .catch(error =>{
            res.status(404).send(error);
        })
}))

//UDP 메세지 보냄 통신
function sendToManager(msg){
    client.send(msg, 0, msg.length, clientPort, clientHost, function(err) {
        if ( err ) return ; 
        console.log("UDP send Msg ")
    });
}

// await Promise 응답해야됨
function responseManager (requestMsg) {
   return new Promise((resolve, reject) => {
        let delay = 100; //ms 단위로 체크
        let waitingTime = 0 //대기 시간 
 
        let timer = setInterval(() => {
            if(requestMsg === JSON.parse(udpResultMsg).cmd ){
                if(requestMsg === 'capture_done'){
                    let base64 = fs.readFileSync('./public/image/capture.jpg' , 'base64')
                    
                    let msg = JSON.parse(udpResultMsg)
                    msg.imageBase64 = base64
                    udpResultMsg = JSON.stringify(msg)
                    // console.log("야이 씨벌 :", JSON.parse(udpResultMsg))
                    // console.log("base64 :",base64) 
                }
                resolve(udpResultMsg)
                clearInterval(timer)
            }else if(waitingTime === 500000){
                reject('No response from Manager')
                clearInterval(timer)
            }else{
                waitingTime = waitingTime + delay
            }
        },delay)
    });
}
 
http.createServer(app).listen(app.get('port'),function(){
    console.log('WebServer Port: ' +app.get('port'))
})
